#include <iomanip>

#include "as_config.h"

#include "config.hh"
#include "error.hh"
#include "file.hh"

//
// ConfigData
//

bool ConfigData::read_in_file(const string & file) {
  if (!StringMap::read_in_file(file))
    throw ae_cant_read(file, 0);
  return true;
}

void ConfigData::replace(const string & key, const string & value) {
  if (value == "false") {
    remove(key);
  } else if (key.substr(0,5) == "dont-") {
    remove(key.substr(5));
  } else {
    StringMap::replace(key, value);
  }
}

ConfigData::ConfigData() {}

ConfigData::ConfigData(const string & file) {
  read_in_file(file);
}

// 
// GlobalConf
//

AspellConfig::AspellConfig() {
  read_in(StringMap());
}

AspellConfig::AspellConfig(const string & override) {
  StringMap temp;
  temp.read_in_string(override);
  read_in(temp);
}
 
AspellConfig::AspellConfig(const StringMap & override) {
  read_in(override);
}

void AspellConfig::read_in(const StringMap & override) {
  AspellConfig & d = *this;

  const char * env = getenv("ASPELL_CONF");
  if (!env) env = "";

  possible_env("HOME", "home");

  //
  // global conf file
  //

  read_in_string(env);
  merge(override);
  insert("conf", "aspell.conf"); // remember insert does not replace
  insert("conf-dir", CONFIGDIR);

  try {
    read_in_file(add_possible_dir(d["conf-dir"], d["conf"]));
  } catch (ae_cant_read) {}

  //
  // personal configuration file
  //

  read_in_string(env);
  merge(override);
  insert("home", "."); // remember insert does not overwrite
  insert("per-conf", ".aspell.conf");

  try {
    read_in_file(add_possible_dir(d["home"],d["per-conf"]));
  } catch (ae_cant_read) {}

  //
  // environment overrides 
  //

  read_in_string(env);

  //
  // local overrides
  //

  merge(override);

  //
  // defaults - remember insert will NOT overwrite
  //

  insert("lang", "english");
  insert("data-dir", DATADIR);
  insert("dict-dir", DICTDIR);
  insert("master", d["lang"]);
  insert("personal", ".aspell." + d["lang"] + ".per");
  insert("repl", ".aspell." + d["lang"] + ".rpl");

  replace("master-path",   add_possible_dir(d["dict-dir"], d["master"]));
  replace("personal-path", add_possible_dir(d["home"],     d["personal"]));
  replace("repl-path",     add_possible_dir(d["home"],     d["repl"]));
}

void AspellConfig::possible_env(const string & env_name, const string & name) {
  const char * v = getenv(env_name.c_str());
  if (v) replace(name,v);
}

struct ASCHelp {
  int args;
  const char * name;
  const char * desc;
  const char * def;
  operator bool () const {return name;}
};

const ASCHelp asc_help[] = {
  {1, "conf", "main configuration file", "aspell.conf"},
  {1, "conf-dir", "location of main configuration file", CONFIGDIR},
  {1, "per-conf", "personal configuration file", ".aspell.conf"},
  {1, "lang", "default language to use", "english"},
  {1, "data-dir", "location of language data files", DATADIR},
  {1, "master", "main word list base name", "<lang>"},
  {1, "dict-dir", "location of the main word list", DICTDIR},
  {1, "personal", "personal word list file name", ".aspell.<lang>.per"},
  {1, "repl", "replacements list file name", ".aspell.<lang>.rpl"},
  {0, "ignore-repl", "ignore commands to store replacement pairs", "false"},
  {0,0,0,0}
};

bool AspellConfig::write_to_stream(ostream & out) const {
  ios::fmtflags f = out.flags();
  out.setf(ios::left);
  out << "# Aspell " VERSION " configuration file" << endl << endl;
  for (int i = 0; asc_help[i]; ++i) {
    out << "## " << asc_help[i].desc << endl
	<< "# default: " << asc_help[i].def << endl
        << setw(15) << asc_help[i].name
	<< (asc_help[i].args == 1 ? lookup(asc_help[i].name) :
	    have(asc_help[i].name) ? "true" : "false")
	<< endl << endl;
  }
  out.flags(f);
  return true;
}

const char * AspellConfig::help(const string & opt) {
  for (int i = 0; asc_help[i]; ++i) 
    if (asc_help[i].name == opt) 
      return asc_help[i].desc;
  return "";
}
